import pandas as pd
import glob
from tqdm import tqdm
# import numpy as np
import collections

def get_beats(data_path,tmp_path,file_index, min_index, annotations_path, dataset_used, samples_before_peak, samples_after_peak, ecg_length, output_path):
    files_list= pd.read_csv(tmp_path+"data_files_list.csv")
    print("Patient ID under progress: ", files_list.iloc[file_index,0].split(".")[0])
    data_filename = data_path + files_list.iloc[file_index,0]
    anno_filename = annotations_path + str(files_list.iloc[file_index,0].split(".")[0]) + "annotations.csv"
    beats_filename1 = output_path + "Beats_with_patientID_" + str(files_list.iloc[file_index,0].split(".")[0]) + ".csv"
    beats_filename2 = output_path + "Beats_NO_patientID_" + str(files_list.iloc[file_index,0].split(".")[0]) + ".csv"
    patient_ID = files_list.iloc[file_index,0].split(".")[0]
    # print(data_filename)
    # print(anno_filename)
    # input("x")
    
    if dataset_used == "kaggle":
        df_data = pd.read_csv(data_filename)
    if dataset_used == "MIT_BIH_official":
        df_data = pd.read_csv(data_filename, header=None)

    max_index = df_data.shape[0]
    # print(df_data)
    # input("X")
    df_anno = pd.read_csv(anno_filename)
    # print(df_anno)
    
    beats_with_label = pd.DataFrame()
    for num, row in tqdm(df_anno.iterrows()):
        # print("num= ",row)
        peak_index = row['Sample']   
        # print(peak_index)
        # print(max_index)
        # print(peak_index > (min_index + samples_before_peak+1))
        # print(peak_index < (max_index - samples_after_peak+1))
        # input("X")
        if peak_index > (min_index + samples_before_peak+1) and peak_index < (max_index - samples_after_peak+1):
            # print("IN LOOP")
            # input("X")
            lower_limit = peak_index - samples_before_peak
            upper_limit = peak_index + samples_after_peak

            if len(row['#'])==1:
                label = ord(row['#'])
            else:
                label = 9999999
            
            # Column Index means MLII
            if dataset_used == "kaggle":
                beat = df_data.iloc[lower_limit:upper_limit,1]
            if dataset_used == "MIT_BIH_official":
                beat = df_data.iloc[lower_limit:upper_limit,0]
            # print(lower_limit)
            # print(upper_limit)
            # print(beat)
            # input("X")

            beat = beat.reset_index(drop=True)

            beat_with_label = pd.concat([beat, pd.DataFrame({0:label},index=[ecg_length]), pd.DataFrame({0:patient_ID},index=[ecg_length+1])],axis=0)
            beat_with_label = beat_with_label.T
            if num == 0:
                beats_with_label = beat_with_label
            else:
                beats_with_label = pd.concat([beats_with_label, beat_with_label],axis=0)
    #input("X")
        # else:
        #     print("Unused peak index = ", peak_index)
        #     input("X")
            
    beats_with_label.to_csv(beats_filename1,index=False)
    beats_with_label_no_patientID = beats_with_label.iloc[:,:-1]
    beats_with_label_no_patientID.to_csv(beats_filename2,index=False)
    
    return beats_with_label, beats_with_label_no_patientID


def create_beats_csv_dataset(data_path, annotations_path, output_path, tmp_path, dataset_used, ecg_length, samples_before_peak, samples_after_peak):
    min_index = 0

    overall_beats_filename = output_path + "All_beats_with_PatientID.csv"
    overall_beats_NO_patientID_filename = output_path + "All_beats_NO_PatientID.csv"
    
    overall_beats_with_label = pd.DataFrame()
    overall_beats_with_label_no_patientID = pd.DataFrame()
    
    #==== Create a csv file containing list of all the subject-wise csv files in the dataset folder ==============##
    data_files=glob.glob1(data_path,'*.csv') #Get list of all .csv files in the current folder
    df=pd.DataFrame(data=data_files)
    df.columns = ['Filename']
    df.to_csv(tmp_path+"data_files_list.csv",index=False) #Write the list to a CSV file
    #=============================================================================================================##
    
        
    files_list=pd.read_csv(tmp_path+"data_files_list.csv")
    for i in range(files_list.shape[0]):
        file_index = i
        beats_with_label, beats_with_label_no_patientID = get_beats(data_path,tmp_path, file_index, min_index, 
                                                                    annotations_path, dataset_used,
                                                                    samples_before_peak, 
                                                                    samples_after_peak, ecg_length, 
                                                                    output_path)

        if i == 0:
            overall_beats_with_label = beats_with_label
            overall_beats_with_label_no_patientID = beats_with_label_no_patientID
        else:
            overall_beats_with_label = pd.concat([overall_beats_with_label, beats_with_label],axis=0)
            overall_beats_with_label_no_patientID = pd.concat([overall_beats_with_label_no_patientID, beats_with_label_no_patientID],axis=0)
        
    overall_beats_with_label.to_csv(overall_beats_filename,index=False)
    overall_beats_with_label_no_patientID.to_csv(overall_beats_NO_patientID_filename,index=False)

def print_stats_for_a_file(csv_file,ecg_length):
    # If not used "header=none", it skips first row.
    df = pd.read_csv(csv_file, header=None)
       
    y = df.iloc[:, ecg_length].values.astype(int)
        
    freq = collections.Counter(y)
    
    print("File being read: ",csv_file)
    print("Class distribution: ")
    for (key, value) in sorted(freq.items()): 
        print(key, " -> ", value)
    print("Total Beats = ", sum(freq.values()))
    print("------------------------------------------------------------------")


def filter_ecg_data(csv_file,list_classes_to_include,ecg_length,output_path):
    # If not used "header=none", it skips first row.
    file_out = pd.read_csv(csv_file)
    
    ## Exclude the non-useful stuff.
    mask = file_out.iloc[:, ecg_length].isin(list_classes_to_include)
    df_filtered = file_out[mask]
    
    save_file_path = output_path + "beats_nsvfq_48p.csv"
    df_filtered.to_csv(save_file_path,index=False, header=None)
    
    print_stats_for_a_file(save_file_path,ecg_length)
  
#==========================================================================================================#

